/* $Id: q931.c,v 1.7 1998/11/15 23:55:17 keil Exp $

 * q931.c       code to decode ITU Q.931 call control messages
 *
 * Author       Jan den Ouden
 *
 * Changelog
 *
 * Pauline Middelink    general improvements
 *
 * Beat Doebeli         cause texts, display information element
 *
 * Karsten Keil         cause texts, display information element for 1TR6
 *
 *
 * $Log: q931.c,v $
 * Revision 1.7  1998/11/15 23:55:17  keil
 * changes from 2.0
 *
 * Revision 1.6  1997/07/27 21:09:44  keil
 * move functions to isdnl3.c
 *
 * Revision 1.5  1997/04/06 22:56:43  keil
 * Some cosmetic changes
 *
 * Revision 1.4  1997/02/09 00:29:11  keil
 * new interface handling, one interface per card
 *
 * Revision 1.3  1997/01/21 22:24:59  keil
 * cleanups
 *
 * Revision 1.2  1996/10/27 22:12:45  keil
 * reporting unknown level 3 protocol ids
 *
 * Revision 1.1  1996/10/13 20:04:56  keil
 * Initial revision
 *
 *
 */


#define __NO_VERSION__
#include "hisax.h"
#include "l3_1tr6.h"


void
iecpy(u_char * dest, u_char * iestart, int ieoffset)
{
	u_char *p;
	int l;

	p = iestart + ieoffset + 2;
	l = iestart[1] - ieoffset;
	while (l--)
		*dest++ = *p++;
	*dest++ = '\0';
}


int QuickHex(char *txt, u_char * p, int cnt)
{
	register int i;
	register char *t = txt;
	register u_char w;

	for (i = 0; i < cnt; i++) {
		*t++ = ' ';
		w = (p[i] >> 4) & 0x0f;
		if (w < 10)
			*t++ = '0' + w;
		else
			*t++ = 'A' - 10 + w;
		w = p[i] & 0x0f;
		if (w < 10)
			*t++ = '0' + w;
		else
			*t++ = 'A' - 10 + w;
	}
	*t++ = 0;
	return (t - txt);
}

void
LogFrame(struct IsdnCardState *cs, u_char * buf, int size)
{
	char *dp;

	if (size < 1)
		return;
	dp = cs->dlog;
	if (size < MAX_DLOG_SPACE / 3 - 10) {
		*dp++ = 'H';
		*dp++ = 'E';
		*dp++ = 'X';
		*dp++ = ':';
		dp += QuickHex(dp, buf, size);
		dp--;
		*dp++ = '\n';
		*dp = 0;
		HiSax_putstatus(cs, NULL, cs->dlog);
	} else
		HiSax_putstatus(cs, "LogFrame: ", "warning Frame too big (%d)", size);
}

void
dlogframe(struct IsdnCardState *cs, struct sk_buff *skb, int dir)
{
	u_char *buf;
	char *dp;
	int size;

	size = skb->len;
	if (size < 3)
		return;

	buf = skb->data;
	dp = cs->dlog;
	if (size < MAX_DLOG_SPACE / 3 - 21) {
		dp += jiftime(dp, jiffies);
		*dp++ = ' ';

		*dp++ = '!';
		*dp++ = 'D';
		*dp++ = 'A';
		*dp++ = 'T';
		*dp++ = 'A';
		*dp++ = dir?'<':'>';
		dp += QuickHex(dp, buf, size);
		dp--;
		*dp++ = '\n';
		*dp = 0;
		HiSax_putstatus(cs, NULL, cs->dlog);
	} else
		HiSax_putstatus(cs, "LogFrame: ", "warning Frame too big (%d)", size);
}
